@@ -1281,9 +1281,37 @@ class BaseLdap(object):
         else:
             return self._id_to_dn_string(object_id)
 
-    @staticmethod
-    def _dn_to_id(dn):
-        return utf8_decode(ldap.dn.str2dn(utf8_encode(dn))[0][0][1])
+    def _dn_to_id(self, dn):
+        # Check if the naming attribute in the DN is the same as keystone's
+        # configured 'id' attribute'.  If so, extract the ID value from the DN
+        if self.id_attr == ldap.dn.str2dn(dn)[0][0][0].lower():
+            return ldap.dn.str2dn(dn)[0][0][1]
+        else:
+            # The 'ID' attribute is NOT in the DN, so we need to perform an
+            # LDAP search to look it up from the user entry itself.
+            with self.get_connection() as conn:
+                search_result = conn.search_s(dn, ldap.SCOPE_BASE)
+
+            if search_result:
+                try:
+                    id_list = search_result[0][1][self.id_attr]
+                except KeyError:
+                    message = ('ID attribute %(id_attr)s not found in LDAP '
+                               'object %(dn)s.') % ({'id_attr': self.id_attr,
+                                                     'dn': search_result})
+                    LOG.warning(message)
+                    raise exception.NotFound(message=message)
+                if len(id_list) > 1:
+                    message = ('In order to keep backward compatibility, in '
+                               'the case of multivalued ids, we are '
+                               'returning the first id %(id_attr) in the '
+                               'DN.') % ({'id_attr': id_list[0]})
+                    LOG.warning(message)
+                return id_list[0]
+            else:
+                message = _('DN attribute %(dn)s not found in LDAP') % (
+                    {'dn': dn})
+                raise exception.NotFound(message=message)
 
     def _ldap_res_to_model(self, res):
         # LDAP attribute names may be returned in a different case than
@@ -1402,20 +1430,31 @@ class BaseLdap(object):
             raise ValueError('"%(attr)s" is not a valid value for'
                              ' "%(attr_name)s"' % {'attr': attr,
                                                    'attr_name': attr_name})
+
+        # consider attr = "cn" and
+        # ldap_result = [{'uid': ['fake_id1']}, , 'cN': ["name"]}]
+        # doing lower case on both user_name_attribute and ldap users
+        # attribute
         result = []
         # consider attr = "cn" and
-        # ldap_result = [{'uid': ['fake_id1']},
-        #                {'uid': ['fake_id2'], 'cn': ['     ']},
-        #                {'uid': ['fake_id3'], 'cn': ['']},
-        #                {'uid': ['fake_id4'], 'cn': []},
-        #                {'uid': ['fake_id5'], 'cn': ["name"]}]
+        # ldap_result = [(u'cn=fake1,o=ex_domain', {'uid': ['fake_id1']}),
+        #                (u'cn=fake2,o=ex_domain', {'uid': ['fake_id2'],
+        #                'cn': ['     ']}),
+        #                (u'cn=fake3,o=ex_domain', {'uid': ['fake_id3'],
+        #                'cn': ['']}),
+        #                (u'cn=fake4,o=ex_domain', {'uid': ['fake_id4'],
+        #                'cn': []}),
+        #                (u'cn=fake5,o=ex_domain', {'uid': ['fake_id5'],
+        #                'cn': ["name"]})]
         for obj in ldap_result:
             # ignore ldap object(user/group entry) which has no attr set
             # in it or whose value is empty list.
-            if obj[1].get(attr):
-                # ignore ldap object whose attr value has empty strings or
-                # contains only whitespaces.
-                if obj[1].get(attr)[0] and obj[1].get(attr)[0].strip():
+            ldap_res_low_keys_dict = {k.lower(): v for k, v in obj[1].items()}
+            result_attr_vals = ldap_res_low_keys_dict.get(attr.lower())
+            # ignore ldap object whose attr value has empty strings or
+            # contains only whitespaces.
+            if result_attr_vals:
+                if result_attr_vals[0] and result_attr_vals[0].strip():
                     result.append(obj)
         # except {'uid': ['fake_id5'], 'cn': ["name"]}, all entries
         # will be ignored in ldap_result
